/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.crop.persistence.serialization;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.EnumSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.crop.model.Crop;
import nl.cloudfarming.client.crop.model.CropManager;
import nl.cloudfarming.client.model.*;
import org.netbeans.api.project.Project;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Wytse Visser
 */
@ServiceProvider(service = CropManager.class)
public class CropManagerImpl extends AbstractEntityManager<Crop> implements CropManager {

    private static final Logger LOGGER = Logger.getLogger(CropManager.class.getName());
    private static final String CROPS_FOLDER = "Crops";

    @Override
    public Set<ItemIdType> getSupportedTypes() {
        return EnumSet.of(ItemIdType.CRP);
    }

    @Override
    public void save(Crop crop) {
        try (ObjectOutputStream oos = new ObjectOutputStream(getCropFile(crop).getOutputStream())) {
            oos.writeObject(crop);
            addToCache(crop);
        } catch (IOException e) {
            LOGGER.log(Level.WARNING, "Could not save crop to file. {0}", e.getMessage());
        }
    }

    private FileObject getCropFile(Crop crop) throws IOException {
        FileObject fileObject = getCropFile(crop.getItemIdNumber());
        if (fileObject == null) {
            fileObject = createCropFile(crop);
        }
        return fileObject;
    }

    private FileObject getCropFile(String itemIdNumber) throws IOException {
        return getCropsFolder().getFileObject(itemIdNumber, Crop.EXT);
    }

    private FileObject createCropFile(Crop crop) throws IOException {
        return getCropsFolder().createData(crop.getItemIdNumber(), Crop.EXT);
    }

    protected FileObject getCropsFolder() throws IOException {
        FileObject folder = FileUtil.getConfigRoot().getFileObject(CROPS_FOLDER);
        if (folder == null) {
            folder = createCropsFolder();
        }
        return folder;
    }

    private FileObject createCropsFolder() throws IOException {
        return FileUtil.getConfigRoot().createFolder(CROPS_FOLDER);
    }

    @Override
    protected Crop load(AgroURI uri, Project project) {
        return load(uri);
    }

    @Override
    protected Crop load(AgroURI uri) {
        Crop crop = null;
        try (ObjectInputStream ois = new ObjectInputStream(getCropFile(uri.getItemIdNumber()).getInputStream())) {
            crop = (Crop) ois.readObject();
            crop = getCachedInstanceFor(crop);
        } catch (IOException | ClassNotFoundException e) {
            LOGGER.log(Level.WARNING, "Could not load crop from file. {0}", e.getMessage());
        }

        return crop;
    }

    @Override
    public void remove(Crop entity) {
        //TODO implement remove
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
